@@ -261,6 +261,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->dup_results_are_passive = GM_ENABLED;
     opt->orphan_host_checks      = GM_ENABLED;
     opt->orphan_service_checks   = GM_ENABLED;
+    opt->orphan_return           = 2;
     opt->accept_clear_results    = GM_DISABLED;
     opt->has_starttime      = FALSE;
     opt->has_finishtime     = FALSE;
@@ -716,8 +717,19 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     /* timeout_return */
     else if ( !strcmp( key, "timeout_return" ) ) {
         opt->timeout_return = atoi( value );
-        if(opt->timeout_return < 0) { opt->timeout_return = 2; }
-        if(opt->timeout_return > 3) { opt->timeout_return = 2; }
+        if(opt->timeout_return < 0 || opt->timeout_return > 3) {
+            gm_log( GM_LOG_INFO, "Warning: unknown timeout_return: %d\n", opt->timeout_return );
+            opt->timeout_return = 2;
+        }
+    }
+
+    /* orphan_return */
+    else if ( !strcmp( key, "orphan_return" ) ) {
+        opt->orphan_return = atoi( value );
+        if(opt->orphan_return < 0 || opt->orphan_return > 3) {
+            gm_log( GM_LOG_INFO, "Warning: unknown orphan_return: %d\n", opt->orphan_return );
+            opt->orphan_return = 2;
+        }
     }
 
     /* perfdata_mode */
@@ -1023,7 +1035,7 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
         gm_log( GM_LOG_DEBUG, "accept clear result:             %s\n", opt->accept_clear_results == GM_ENABLED ? "yes" : "no");
     }
     gm_log( GM_LOG_DEBUG, "transport mode:                  %s\n", opt->encryption == GM_ENABLED ? "aes-256+base64" : "base64 only");
-    gm_log( GM_LOG_DEBUG, "use uniq jobs:                   %s\n", opt->use_uniq_jobs == GM_ENABLED ? "overwrite" : "append");
+    gm_log( GM_LOG_DEBUG, "use uniq jobs:                   %s\n", opt->use_uniq_jobs == GM_ENABLED ? "yes" : "no");
 
     gm_log( GM_LOG_DEBUG, "--------------------------------\n" );
     return;
@@ -1093,8 +1105,10 @@ int read_keyfile(mod_gm_opt_t *opt) {
         free(opt->crypt_key);
     opt->crypt_key = malloc(GM_BUFFERSIZE);
 
-    if(!fgets(opt->crypt_key, 33, fp))
+    if(!fgets(opt->crypt_key, 33, fp)) {
+        fclose(fp);
         return(GM_ERROR);
+    }
     fclose(fp);
     rtrim(opt->crypt_key);
 
@@ -1200,16 +1214,17 @@ int free_job(gm_job_t *job) {
     return(GM_OK);
 }
 
-
 /* verify if a pid is alive */
 int pid_alive(int pid) {
+    int status;
+
     if(pid < 0) { pid = -pid; }
 
     /* 1/-1 are undefined pids in our case */
     if(pid == 1)
-        return TRUE;
+        return FALSE;
 
-    /* send kill 0 to verify the proc is alive */
+    /* send kill 0 to verify the process still exists */
     if(kill(pid, 0) == 0) {
         return TRUE;
     }
