@@ -39,9 +39,9 @@ void *start_gearmand(void*data) {
         snprintf(port, 30, "--port=%d", GEARMAND_TEST_PORT);
         /* for newer gearman versions */
         if(atof(gearman_version()) >= 0.27) {
-            execlp("gearmand", "gearmand", "--threads=10", "--job-retries=0", port, "--verbose=DEBUG", "--log-file=/tmp/gearmand.log" , (char *)NULL);
+            execlp("gearmand", "gearmand", "--listen=127.0.0.1", "--threads=10", "--job-retries=0", port, "--verbose=DEBUG", "--log-file=/tmp/gearmand.log" , (char *)NULL);
         } else if(atof(gearman_version()) > 0.14) {
-            execlp("gearmand", "gearmand", "--threads=10", "--job-retries=0", port, "--verbose=999", "--log-file=/tmp/gearmand.log" , (char *)NULL);
+            execlp("gearmand", "gearmand", "--listen=127.0.0.1", "--threads=10", "--job-retries=0", port, "--verbose=999", "--log-file=/tmp/gearmand.log" , (char *)NULL);
         } else {
             /* for gearman 0.14 */
             execlp("gearmand", "gearmand", "-t 10", "-j 0", port, (char *)NULL);
@@ -65,7 +65,7 @@ void *start_worker(void*data) {
     if(pid == 0) {
         sid = setsid();
         char options[150];
-        snprintf(options, 150, "server=localhost:%d", GEARMAND_TEST_PORT);
+        snprintf(options, 150, "server=127.0.0.1:%d", GEARMAND_TEST_PORT);
         char logf[150];
         snprintf(logf, 150, "logfile=%s", worker_logfile);
         if(key != NULL) {
@@ -200,10 +200,10 @@ void create_modules() {
     ok(create_client( mod_gm_opt->server_list, &client ) == GM_OK, "created test client");
 
     ok(create_worker( mod_gm_opt->server_list, &worker ) == GM_OK, "created test worker");
-    //gearman_worker_add_options(&worker, GEARMAN_WORKER_NON_BLOCKING);
-    gearman_worker_set_timeout(&worker, 5000);
     ok(worker_add_function( &worker, GM_DEFAULT_RESULT_QUEUE, get_results ) == GM_OK, "added result worker");
     ok(worker_add_function( &worker, "dummy", dummy ) == GM_OK, "added dummy worker");
+    //gearman_worker_add_options(&worker, GEARMAN_WORKER_NON_BLOCKING);
+    gearman_worker_set_timeout(&worker, 5000);
     return;
 }
 
@@ -214,7 +214,7 @@ void do_result_work(int nr) {
     int x = 0;
     for(x=0;x<nr;x++) {
         ret = gearman_worker_work( &worker );
-        //ok(ret == GEARMAN_SUCCESS, 'got valid job' );
+        ok(ret == GEARMAN_SUCCESS, "got valid job from result queue" );
     }
     return;
 }
@@ -240,7 +240,8 @@ void check_logfile(char *logfile, int mode) {
     line = malloc(GM_BUFFERSIZE);
     while(fgets(line, GM_BUFFERSIZE, fp) != NULL) {
         if(strstr(line, "ERROR") != NULL) {
-            mode == 2 && diag("logfile: %s", line);
+            if(mode == 2)
+                diag("logfile: %s", line);
             errors++;
         }
     }
@@ -268,6 +269,34 @@ void check_logfile(char *logfile, int mode) {
     return;
 }
 
+/* diag queues */
+void diag_queues(void);
+void diag_queues() {
+    char * message = NULL;
+    char * version = NULL;
+    int rc, x;
+    mod_gm_server_status_t *stats;
+
+    // print some debug info
+    stats = malloc(sizeof(mod_gm_server_status_t));
+    stats->function_num = 0;
+    stats->worker_num   = 0;
+    rc = get_gearman_server_data(stats, &message, &version, "127.0.0.1", GEARMAND_TEST_PORT);
+    diag("get_gearman_server_data:  rc: %d\n", rc);
+    diag("get_gearman_server_data: msg: %s\n", message);
+    diag("get_gearman_server_data: ver: %s\n", version);
+    if( rc == STATE_OK ) {
+        diag("%-35s %-9s %-9s\n", "queue", "waiting", "running");
+        for(x=0; x<stats->function_num;x++) {
+            diag("%-35s %-9d %-9d\n", stats->function[x]->queue, stats->function[x]->waiting, stats->function[x]->running);
+        }
+    }
+    free(message);
+    free(version);
+    free_mod_gm_status_server(stats);
+    return;
+}
+
 /* wait till the given queue is empty */
 void wait_for_empty_queue(char *queue, int timeout);
 void wait_for_empty_queue(char *queue, int timeout) {
@@ -283,10 +312,9 @@ void wait_for_empty_queue(char *queue, int timeout) {
         stats = malloc(sizeof(mod_gm_server_status_t));
         stats->function_num = 0;
         stats->worker_num   = 0;
-        rc = get_gearman_server_data(stats, &message, &version, "localhost", GEARMAND_TEST_PORT);
+        rc = get_gearman_server_data(stats, &message, &version, "127.0.0.1", GEARMAND_TEST_PORT);
         if( rc == STATE_OK ) {
             for(x=0; x<stats->function_num;x++) {
-                //diag("%s %d %d\n", stats->function[x]->queue, stats->function[x]->waiting, stats->function[x]->running);
                 if(stats->function[x]->waiting == 0 &&
                    stats->function[x]->running == 0 &&
                    !strcmp( stats->function[x]->queue, queue )
@@ -302,6 +330,11 @@ void wait_for_empty_queue(char *queue, int timeout) {
     }
 
     ok(tries < timeout, "queue %s empty after %d seconds", queue, tries);
+
+    if(tries >= timeout) {
+        diag_queues();
+    }
+
     return;
 }
 
@@ -310,18 +343,39 @@ char* my_tmpfile() {
     char *sfn = strdup("/tmp/modgm.XXXXXX");
     int fd = -1;
     if ((fd = mkstemp(sfn)) == -1) {
-       fprintf(stderr, "%s: %s\n", sfn, strerror(errno));
-       return (NULL);
+        fprintf(stderr, "%s: %s\n", sfn, strerror(errno));
+        free(sfn);
+        return (NULL);
     }
     close(fd);
     return sfn;
 }
 
+void check_no_worker_running(char*);
+void check_no_worker_running(char* worker_logfile) {
+    char cmd[150];
+    char *result, *error;
+    int rrc;
+
+    // ensure no worker are running anymore
+    char *username=getenv("USER");
+    snprintf(cmd, 150, "ps -efl 2>/dev/null | grep -v grep | grep '%s' | grep mod_gearman_worker", username);
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    ok(rrc == 1, "no worker running anymore");
+    like(result, "^\\s*$", "ps output should be empty");
+    like(error, "^\\s*$", "ps error output should be empty");
+    if(rrc != 1) {
+        check_logfile(worker_logfile, 3);
+    }
+    return;
+}
+
+
 /* main tests */
 int main (int argc, char **argv, char **env) {
     argc = argc; argv = argv; env  = env;
     int status, chld, rc;
-    int tests = 92;
+    int tests = 127;
     int rrc;
     char cmd[150];
     char *result, *error, *message, *output;
@@ -338,7 +392,7 @@ int main (int argc, char **argv, char **env) {
 #endif
 
     char options[150];
-    snprintf(options, 150, "--server=localhost:%d", GEARMAND_TEST_PORT);
+    snprintf(options, 150, "--server=127.0.0.1:%d", GEARMAND_TEST_PORT);
     ok(parse_args_line(mod_gm_opt, options, 0) == 0, "parse_args_line()");
     mod_gm_opt->debug_level = GM_LOG_ERROR;
 
@@ -350,20 +404,24 @@ int main (int argc, char **argv, char **env) {
 
     /* first fire up a gearmand server and one worker */
     start_gearmand((void*)NULL);
+    sleep(2);
     start_worker((void*)NULL);
+    sleep(2);
 
     /* wait one second and catch died procs */
-    sleep(1);
-        while((chld = waitpid(-1, &status, WNOHANG)) != -1 && chld > 0) {
+    while((chld = waitpid(-1, &status, WNOHANG)) != -1 && chld > 0) {
         diag( "waitpid() %d exited with %d\n", chld, status);
         status = 0;
     }
 
-    if(!ok(gearmand_pid > 0, "'gearmand started with pid: %d", GEARMAND_TEST_PORT, gearmand_pid)) {
+    if(!ok(gearmand_pid > 0, "'gearmand started with port %d and pid: %d", GEARMAND_TEST_PORT, gearmand_pid)) {
         diag("make sure gearmand is in your PATH. Common locations are /usr/sbin or /usr/local/sbin");
         exit( EXIT_FAILURE );
     }
     if(!ok(pid_alive(gearmand_pid) == TRUE, "gearmand alive")) {
+        check_logfile("/tmp/gearmand.log", 3);
+        kill(gearmand_pid, SIGTERM);
+        kill(worker_pid, SIGTERM);
         exit( EXIT_FAILURE );
     }
     if(!ok(worker_pid > 0, "worker started with pid: %d", worker_pid))
@@ -371,6 +429,7 @@ int main (int argc, char **argv, char **env) {
     if(!ok(pid_alive(worker_pid) == TRUE, "worker alive")) {
         check_logfile(worker_logfile, 3);
         kill(gearmand_pid, SIGTERM);
+        kill(worker_pid, SIGTERM);
         exit( EXIT_FAILURE );
     }
 
@@ -384,36 +443,63 @@ int main (int argc, char **argv, char **env) {
 
     /* send big job */
     send_big_jobs(GM_ENCODE_ONLY);
+    //diag_queues();
+    wait_for_empty_queue("eventhandler", 20);
+    wait_for_empty_queue("service", 20);
+    //diag_queues();
     do_result_work(1);
+    //diag_queues();
+    wait_for_empty_queue(GM_DEFAULT_RESULT_QUEUE, 5);
 
     /*****************************************
      * test check
      */
+    //diag_queues();
     test_servicecheck(GM_ENCODE_ONLY, "./t/crit.pl");
+    //diag_queues();
+    wait_for_empty_queue("eventhandler", 20);
+    wait_for_empty_queue("service", 5);
+    //diag_queues();
     do_result_work(1);
+    //diag_queues();
+    wait_for_empty_queue(GM_DEFAULT_RESULT_QUEUE, 5);
+    //diag_queues();
     like(last_result, "test plugin CRITICAL", "stdout output from ./t/crit.pl");
     like(last_result, "some errors on stderr", "stderr output from ./t/crit.pl");
 
     /*****************************************
      * test check2
      */
+    //diag_queues();
     test_servicecheck(GM_ENCODE_ONLY, "./t/both");
+    //diag_queues();
+    wait_for_empty_queue("eventhandler", 20);
+    wait_for_empty_queue("service", 5);
+    //diag_queues();
     do_result_work(1);
+    //diag_queues();
+    wait_for_empty_queue(GM_DEFAULT_RESULT_QUEUE, 5);
     like(last_result, "stdout output", "stdout output from ./t/both");
     like(last_result, "stderr output", "stderr output from ./t/both");
 
     /* try to send some data with base64 only */
+    //diag_queues();
     test_eventhandler(GM_ENCODE_ONLY);
+    //diag_queues();
     test_servicecheck(GM_ENCODE_ONLY, NULL);
-    do_result_work(1);
+    //diag_queues();
     wait_for_empty_queue("eventhandler", 20);
     wait_for_empty_queue("service", 5);
+    //diag_queues();
+    do_result_work(1);
+    //diag_queues();
     wait_for_empty_queue(GM_DEFAULT_RESULT_QUEUE, 5);
     sleep(1);
     kill(worker_pid, SIGTERM);
     waitpid(worker_pid, &status, 0);
-    ok(status == 0, "worker exited with exit code %d", real_exit_code(status));
+    ok(status == 0, "worker (%d) exited with exit code %d", worker_pid, real_exit_code(status));
     status = 0;
+    check_no_worker_running(worker_logfile);
     check_logfile(worker_logfile, 0);
 
     char * test_keys[] = {
@@ -436,24 +522,24 @@ int main (int argc, char **argv, char **env) {
 
         test_eventhandler(GM_ENCODE_AND_ENCRYPT);
         test_servicecheck(GM_ENCODE_AND_ENCRYPT, NULL);
-        do_result_work(1);
-
         wait_for_empty_queue("eventhandler", 20);
         wait_for_empty_queue("service", 5);
+        do_result_work(1);
         wait_for_empty_queue(GM_DEFAULT_RESULT_QUEUE, 5);
         sleep(1);
 
         kill(worker_pid, SIGTERM);
         waitpid(worker_pid, &status, 0);
-        ok(status == 0, "worker exited with exit code %d", real_exit_code(status));
+        ok(status == 0, "worker (%d) exited with exit code %d", worker_pid, real_exit_code(status));
         status = 0;
+        check_no_worker_running(worker_logfile);
         check_logfile(worker_logfile, 0);
     }
 
     /*****************************************
      * send_gearman
      */
-    snprintf(cmd, 150, "./send_gearman --server=localhost:%d --key=testtest --host=test --service=test --message=test --returncode=0", GEARMAND_TEST_PORT);
+    snprintf(cmd, 150, "./send_gearman --server=127.0.0.1:%d --key=testtest --host=test --service=test --message=test --returncode=0", GEARMAND_TEST_PORT);
     rrc = real_exit_code(run_check(cmd, &result, &error));
     cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
     like(result, "^\\s*$", "output from ./send_gearman");
@@ -463,7 +549,7 @@ int main (int argc, char **argv, char **env) {
     /*****************************************
      * send_multi
      */
-    snprintf(cmd, 150, "./send_multi --server=localhost:%d --host=blah < t/data/send_multi.txt", GEARMAND_TEST_PORT);
+    snprintf(cmd, 150, "./send_multi --server=127.0.0.1:%d --host=blah < t/data/send_multi.txt", GEARMAND_TEST_PORT);
     rrc = real_exit_code(run_check(cmd, &result, &error));
     cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
     like(result, "send_multi OK: 2 check_multi child checks submitted", "output from ./send_multi");
@@ -473,7 +559,7 @@ int main (int argc, char **argv, char **env) {
     /*****************************************
      * check_gearman
      */
-    snprintf(cmd, 150, "./check_gearman -H localhost:%d -s check -a -q worker_test", GEARMAND_TEST_PORT);
+    snprintf(cmd, 150, "./check_gearman -H 127.0.0.1:%d -s check -a -q worker_test", GEARMAND_TEST_PORT);
     rrc = real_exit_code(run_check(cmd, &result, &error));
     cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
     like(result, "check_gearman OK - sending background job succeded", "output from ./check_gearman");
@@ -486,7 +572,7 @@ int main (int argc, char **argv, char **env) {
     free_worker(&worker);
 
     /* shutdown gearmand */
-    rc = send2gearmandadmin("shutdown\n", "localhost", GEARMAND_TEST_PORT, &output, &message);
+    rc = send2gearmandadmin("shutdown\n", "127.0.0.1", GEARMAND_TEST_PORT, &output, &message);
     ok(rc == 0, "rc of send2gearmandadmin %d", rc);
     like(output, "OK", "output contains OK");
     free(message);
@@ -497,7 +583,7 @@ int main (int argc, char **argv, char **env) {
         waitpid(gearmand_pid, &status, WNOHANG);
         if(pid_alive(gearmand_pid) == FALSE) {
             todo();
-            ok(status == 0, "gearmand exited with: %d", real_exit_code(status));
+            ok(status == 0, "gearmand (%d) exited with: %d", gearmand_pid, real_exit_code(status));
             endtodo;
             break;
         }
@@ -508,7 +594,7 @@ int main (int argc, char **argv, char **env) {
         /* kill it the hard way */
         kill(gearmand_pid, SIGTERM);
         waitpid(gearmand_pid, &status, 0);
-        ok(status == 0, "gearmand exited with exit code %d", real_exit_code(status));
+        ok(status == 0, "gearmand (%d) exited with exit code %d", gearmand_pid, real_exit_code(status));
         status = 0;
         ok(false, "gearmand had to be killed!");
     }
@@ -518,7 +604,8 @@ int main (int argc, char **argv, char **env) {
 
     kill(worker_pid, SIGTERM);
     waitpid(worker_pid, &status, 0);
-    ok(status == 0, "worker exited with exit code %d", real_exit_code(status));
+    ok(status == 0, "worker (%d) exited with exit code %d", worker_pid, real_exit_code(status));
+    check_no_worker_running(worker_logfile);
     check_logfile(worker_logfile, 2);
     status = 0;
 
